@@ -26,6 +26,10 @@ class ScriptInfo(DictObject):
     def __init__(self, *args, testcase='', **kwargs):
         super().__init__(*args, **kwargs)
         self.testcase = testcase
+        self.variables = DictObject(
+            test_resource_var='test_resource', test_resource_ref='',
+            test_data_var='test_data'
+        )
 
     def get_class_name(self):
         node = self.get(self.testcase)
@@ -41,6 +45,15 @@ class ScriptInfo(DictObject):
             else:
                 return 'test_step'
 
+    def clear_devices_vars(self):
+        setattr(self, 'devices_vars', DictObject())
+
+    def reset_global_vars(self):
+        self.variables = DictObject(
+            test_resource_var='test_resource', test_resource_ref='',
+            test_data_var='test_data'
+        )
+
 
 SCRIPTINFO = ScriptInfo()
 
@@ -81,6 +94,10 @@ class Statement:
     def name(self):
         return self._name
 
+    @property
+    def children(self):
+        return self._children
+
     @property
     def level(self):
         return self._level
@@ -150,28 +167,44 @@ class Statement:
                         self._spacers = match.group('spacers')
                         length = len(self._spacers)
                         if length == 0:
-                            self._level = 0
+                            self.set_level(level=0)
                         else:
                             if self.parent:
                                 chk_lst = ['setup', 'cleanup', 'teardown', 'section']
                                 if self.parent.name in chk_lst:
-                                    self._level = 1
+                                    self.set_level(level=1)
                                 else:
-                                    self._level += 1
+                                    self.increase_level()
                             else:
                                 if self._prev_spacers > self._spacers:
-                                    self._level += 1
+                                    self.increase_level()
 
                     self._prev_spacers = self._spacers
                     self._stmt_data = line
                     self._remaining_data = '\n'.join(lst[index+1:])
 
                     if self.is_base_statement:
-                        self._level = 0
+                        self.set_level(level=0)
                         self._spacers = ''
 
                     return
 
+    def add_child(self, child):
+        if isinstance(child, Statement):
+            self._children.append(child)
+            if isinstance(child.parent, Statement):
+                child.set_level(level=self.level+1)
+
+    def set_level(self, level=0):
+        self._level = level
+
+    def increase_level(self):
+        self.set_level(level=self.level+1)
+
+    def update_level_from_parent(self):
+        if isinstance(self.parent, Statement):
+            self.set_level(level=self.parent.level+1)
+
     def get_next_statement_data(self):
         for line in self.remaining_data.splitlines():
             if line.strip():
@@ -233,11 +266,11 @@ class Statement:
         is_logger = getattr(self, 'is_logger', False)
         func_name = 'self.logger.info' if is_logger else 'print'
         if self.framework == FWTYPE.UNITTEST:
-            stmt = '{}({!r})'.format(func_name, message)
+            stmt = '%s(%r)' % (func_name, message)
         elif self.framework == FWTYPE.PYTEST:
-            stmt = '{}({!r})'.format(func_name, message)
+            stmt = '%s(%r)' % (func_name, message)
         else:   # i.e ROBOTFRAMEWORK
-            stmt = 'log   {}'.format(message)
+            stmt = 'log   %s' % message
 
         level = self.parent.level + 1 if self.parent else self.level
         stmt = self.indent_data(stmt, level)
@@ -252,10 +285,10 @@ class Statement:
             fmt1 = 'self.assertTrue(True == %s)'
             fmt2 = 'total_count = len(result)\nself.assertTrue(total_count == %s)'
         elif self.framework == FWTYPE.PYTEST:
-            fmt1 = 'assert True == {}'
+            fmt1 = 'assert True == %s'
             fmt2 = 'total_count = len(result)\nassert total_count == %s'
         else:   # i.e ROBOTFRAMEWORK
-            fmt1 = 'should be true   True == {}'
+            fmt1 = 'should be true   True == %s'
             fmt2 = ('${total_count}=   get length ${result}\nshould be '
                     'true   ${result} == %s')
 
@@ -320,7 +353,7 @@ class SetupStatement(Statement):
         else:   # i.e ROBOTFRAMEWORK
             lst.append('setup')
 
-        for child in self._children:
+        for child in self.children:
             lst.append(child.snippet)
 
         level = 0 if self.framework == FWTYPE.ROBOTFRAMEWORK else 1
@@ -333,19 +366,18 @@ class SetupStatement(Statement):
             self._is_parsed = True
             if self.is_next_statement_children():
                 node = self.create_child(self)
-                node and self._children.append(node)
+                self.add_child(node)
                 while node and node.is_next_statement_sibling():
                     node = self.create_child(node)
-                    node and self._children.append(node)
-                if self._children:
+                    self.add_child(node)
+                if self.children:
                     last_child = self._children[-1]
                     self._remaining_data = last_child.remaining_data
-            if not self._children:
+            if not self.children:
                 kwargs = dict(framework=self.framework, indentation=self.indentation)
                 data = 'dummy_pass - Dummy Setup'
                 dummy_stmt = DummyStatement(data, **kwargs, parent=self)
-                dummy_stmt._level = 1
-                self._children.append(dummy_stmt)
+                self.add_child(dummy_stmt)
         else:
             self._is_parsed = False
 
@@ -366,10 +398,10 @@ class SetupStatement(Statement):
         # node.next = other
         if node.name == 'setup':
             other.parent = node
-            other._level = other.parent.level + 1
+            other.update_level_from_parent()
         else:
             other.parent = node.parent
-            other._level = other.parent.level + 1
+            other.update_level_from_parent()
         return other
 
 
@@ -387,10 +419,10 @@ class ConnectDataStatement(Statement):
             return ''
 
         if self.framework == FWTYPE.ROBOTFRAMEWORK:
-            fmt = "${%s}=   connect data   filename='%s'\nset global variable   %s"
+            fmt = "${%s}=   connect data   filename=%s\nset global variable   ${%s}"
             stmt = fmt % (self.var_name, self.test_resource_ref, self.var_name)
         else:
-            fmt = "self.%s = ta.connect_data(filename='%s')"
+            fmt = "self.%s = ta.connect_data(filename=%r)"
             stmt = fmt % (self.var_name, self.test_resource_ref)
 
         level = self.parent.level + 1 if self.parent else self.level
@@ -399,7 +431,7 @@ class ConnectDataStatement(Statement):
         return stmt
 
     def parse(self):
-        pattern = r'(?i) +connect +data +(?P<capture_data>.+)'
+        pattern = r'(?i) *connect +data +(?P<capture_data>.+)'
         match = re.match(pattern, self.statement_data)
         if match:
             capture_data = match.group('capture_data').strip()
@@ -455,13 +487,13 @@ class UseTestCaseStatement(Statement):
         if not self.is_parsed:
             return ''
 
-        test_resource_var = SCRIPTINFO.variables.test_resource_var  # noqa
+        test_resource_var = SCRIPTINFO.variables.get('test_resource_var', 'test_resource')
 
         if self.framework == FWTYPE.ROBOTFRAMEWORK:
-            fmt = "${%s}=  use testcase   %s  device='%s'\nset global variable   %s"
+            fmt = "${%s}=  use testcase   ${%s}  testcase=%s\nset global variable   ${%s}"
             stmt = fmt % (self.var_name, test_resource_var, self.test_name, self.var_name)
         else:
-            fmt = "self.%s = ta.use_testcase(self.%s, device='%s')"
+            fmt = "self.%s = ta.use_testcase(self.%s, testcase=%r)"
             stmt = fmt % (self.var_name, test_resource_var, self.test_name)
 
         level = self.parent.level + 1 if self.parent else self.level
@@ -470,7 +502,7 @@ class UseTestCaseStatement(Statement):
         return stmt
 
     def parse(self):
-        pattern = r'(?i) +use +testcase +(?P<capture_data>[a-z0-9].+)'
+        pattern = r'(?i) *use +testcase +(?P<capture_data>[a-z0-9].+)'
         match = re.match(pattern, self.statement_data)
         if not match:
             self._is_parsed = False
@@ -525,11 +557,11 @@ class ConnectDeviceStatement(Statement):
         lst = []
         for var_name, device_name in self.devices_vars.items():
             if self.framework == FWTYPE.ROBOTFRAMEWORK:
-                fmt = "${%s}=  connect device   %s   name='%s'\nset global variable   %s"
+                fmt = "${%s}=   connect device   ${%s}   name=%s\nset global variable   ${%s}"
                 stmt = fmt % (var_name, test_resource_var, device_name, var_name)
 
             else:
-                fmt = "self.%s = ta.connect_device(self.%s, name='%s')"
+                fmt = "self.%s = ta.connect_device(self.%s, name=%r)"
                 stmt = fmt % (var_name, test_resource_var, device_name)
             lst.append(stmt)
 
@@ -539,7 +571,7 @@ class ConnectDeviceStatement(Statement):
         return connect_device_statements
 
     def parse(self):
-        pattern = r'(?i) +connect +device +(?P<devices_info>.+) *$'
+        pattern = r'(?i) *connect +device +(?P<devices_info>.+) *$'
         match = re.match(pattern, self.statement_data)
         if not match:
             self._is_parsed = False
